/*
 * Created on Jan 25, 2005
 */
package com.alwara.graphLayout.test;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JPanel;


public class GraphPanel extends JPanel implements Runnable {
   private static final long serialVersionUID = -1L;

   private final Graph graph_;

   char lowest = 'a';

   private static final int SLEEP_TIME = 200;

   private static final int Y_OFFSET = 100;

   private static final int X_OFFSET = 100;

   private static final int normalEdgeLength = 70;

   int centerX = this.getWidth() / 2;

   int centerY = this.getHeight() / 2;

   public GraphPanel(Graph graph) {
      this.graph_ = graph;
      add(new JLabel("Data to show"));

      Thread refreshThread = new Thread(this);
      refreshThread.start();

   }

   public void run() {
      while (true) {
         try {
            Thread.sleep(SLEEP_TIME);
         } catch (InterruptedException e) {
            e.printStackTrace();
         }
         refreshLayout();
      }
   }

   public void refreshLayout() {
      for (Iterator iter = graph_.getEdges().iterator(); iter.hasNext();) {
         Edge edge = (Edge) iter.next();
         if (length(edge) > (normalEdgeLength * edge.getWeight())) {
            reduceLength(edge);
         } else {
            increaseLength(edge);
         }
      }

      repaint();
   }

   int ratioIncrease = 20;

   /**
    * Move the one that is closer to the center to further away from the other node.
    */
   private void increaseLength(Edge edge) {
      Node firstNode = edge.getFirstNode();
      Node secondNode = edge.getSecondNode();
      if (closerToCenter(firstNode, secondNode) > 0) {
         firstNode.xLoc = firstNode.xLoc
               + ((firstNode.xLoc - secondNode.xLoc) / ratioIncrease);
         firstNode.yLoc = firstNode.yLoc
               + ((firstNode.yLoc - secondNode.yLoc) / ratioIncrease);
      } else {
         secondNode.xLoc = secondNode.xLoc
               + ((secondNode.xLoc - firstNode.xLoc) / ratioIncrease);
         secondNode.yLoc = secondNode.yLoc
               + ((secondNode.yLoc - firstNode.yLoc) / ratioIncrease);
      }
   }

   /**
    * Move the one that is farther from the center to closer to the other node.
    */
   private void reduceLength(Edge edge) {
      Node firstNode = edge.getFirstNode();
      Node secondNode = edge.getSecondNode();
      if (closerToCenter(firstNode, secondNode) > 0) {
         firstNode.xLoc = firstNode.xLoc
               - ((firstNode.xLoc - secondNode.xLoc) / ratioIncrease);
         firstNode.yLoc = firstNode.yLoc
               - ((firstNode.yLoc - secondNode.yLoc) / ratioIncrease);
      } else {
         secondNode.xLoc = secondNode.xLoc
               - ((secondNode.xLoc - firstNode.xLoc) / ratioIncrease);
         secondNode.yLoc = secondNode.yLoc
               - ((secondNode.yLoc - firstNode.yLoc) / ratioIncrease);
      }
   }

   /**
    * Return a +ve number if firstNode is closer else return a -ve number.
    */
   private int closerToCenter(Node firstNode, Node secondNode) {
      int node1Distance = distanceFromPoint(firstNode, centerX, centerY);
      int node2Distance = distanceFromPoint(secondNode, centerX, centerY);

      return (node2Distance - node1Distance);
   }

   private static int distanceFromPoint(Node firstNode, int centerX, int centerY) {
      int node1Distance = (int) Math.hypot(firstNode.xLoc - centerX,
            firstNode.yLoc - centerY);
      return node1Distance;
   }

   public static int length(Edge edge) {
      Node node1 = edge.getFirstNode();
      Node node2 = edge.getSecondNode();

      int xDistance = node1.xLoc - node2.xLoc;
      int yDistance = node1.yLoc - node2.yLoc;

      int length = (int) (Math.hypot(xDistance, yDistance));
      return length;
   }

   public void paint(Graphics g) {
      int i = 0;
      g.clearRect(0, 0, this.getWidth(), this.getHeight());

      for (Iterator iter = graph_.getNodes().iterator(); iter.hasNext();) {
         Node node = (Node) iter.next();
         g.setColor(Color.blue);
         char nodeName = (char) (lowest + i++);
         g.drawString(Character.toString(nodeName), node.xLoc + X_OFFSET,
               node.yLoc + Y_OFFSET);
      }

      for (Iterator iter = graph_.getEdges().iterator(); iter.hasNext();) {
         Edge edge = (Edge) iter.next();

         Color oldColor = g.getColor();
         g.setColor(Color.blue);

         Node firstNode = edge.getFirstNode();
         Node secondNode = edge.getSecondNode();
         g.drawLine(firstNode.xLoc + X_OFFSET, firstNode.yLoc + Y_OFFSET,
               secondNode.xLoc + X_OFFSET, secondNode.yLoc + Y_OFFSET);
         
         int weightX = firstNode.xLoc + X_OFFSET + secondNode.xLoc + X_OFFSET;
         int weightY = firstNode.yLoc + Y_OFFSET + secondNode.yLoc + Y_OFFSET;
            
         g.drawString("" + edge.getWeight(), weightX/2, weightY/2);

         g.setColor(oldColor);
      }
   }
}
