/*
 * Created on Jan 3, 2005
 */
package com.alwara.uml.clazz;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.gef.ui.rulers.RulerComposite;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.core.util.Util;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;

import com.alwara.uml.clazz.model.ClazzDiagramModel;
import com.alwara.uml.clazz.model.ClazzModel;
import com.alwara.uml.clazz.model.ClazzModelFactory;
import com.alwara.uml.clazz.view.ClazzDropListener;
import com.alwara.uml.model.AbstractConnectionModel;
import com.alwara.uml.model.ArrowConnectionModel;
import com.alwara.uml.model.ImplementsConnectionModel;
import com.alwara.uml.model.LineConnectionModel;
import com.alwara.uml.utils.InspectJavaElementUtils;
import com.alwara.uml.utils.Logger;
import com.alwara.uml.view.IconImage;

/**
 * @author Virender Singh Dogra
 */
public class ClazzDiagramEditor extends GraphicalEditorWithPalette {
   //public class ClassDiagramEditor extends GraphicalEditor {
   Logger logger_ = Logger.getLogger(ClazzDiagramEditor.class);

   private static final String DRAWER_NAME = "Class Diagram",
                               CLAZZ_EXTENSION = ".clazz",
                               JAVA_EXTENSION = Util.defaultJavaExtension();

   RulerComposite rulerComp_;
   ClazzDiagramModel diagramModel;

   /** Constructor  */
   public ClazzDiagramEditor() {
      setEditDomain(new DefaultEditDomain(this));
   }

   public void init(IEditorSite site, IEditorInput input) throws PartInitException {
      super.init(site, input);

      diagramModel = new ClazzDiagramModel();

      if (isJavaFile(input)) {
         String fullfilename = ((IFileEditorInput) input).getFile().getLocation().makeAbsolute().toString();
         ICompilationUnit compilationUnit = InspectJavaElementUtils.getCompilationUnit(fullfilename);

         //show the main clazz
         ClazzModel mainClazz = new ClazzModel(compilationUnit);
         deleteThisSetMainClazz(compilationUnit);
         
         int xBegin = 0, yBegin = 0;
         
         mainClazz.setConstraint(new Rectangle(xBegin, yBegin, -1, -1));
         diagramModel.addClazz(mainClazz);

         //show the class it extends
         ICompilationUnit superClass = getExtendsClass(compilationUnit);

         if (superClass != null) {
            ClazzModel superClazz = new ClazzModel(superClass);
            superClazz.setConstraint(new Rectangle(xBegin += 100, yBegin, -1, -1));
            diagramModel.addClazz(superClazz);

            //show the extends connection
            AbstractConnectionModel extendsConnection = new ArrowConnectionModel();
            mainClazz.addSourceConnection(extendsConnection);
            superClazz.addTargetConnection(extendsConnection);
         }

         ICompilationUnit[] superInterfacesUnits = getInterfaceUnits(compilationUnit);
         for (int i = 0; i < superInterfacesUnits.length; i++) {
            ClazzModel interfaceClazzModel = new ClazzModel(superInterfacesUnits[i]);
            interfaceClazzModel.setConstraint(new Rectangle(xBegin += 100, yBegin += 100, -1, -1));
            diagramModel.addClazz(interfaceClazzModel);

            //show the implements connection
            AbstractConnectionModel interfaceConnection = new ImplementsConnectionModel();
            mainClazz.addSourceConnection(interfaceConnection);
            interfaceClazzModel.addTargetConnection(interfaceConnection);
         }
      } else if (isClazzDiagram(input)) {

      } else {
         assert false : "Invalid branching";
      }

   }

   ICompilationUnit deleteThisMainClazz;
   
   /**
    * @param mainClazz
    */
   private void deleteThisSetMainClazz(ICompilationUnit mainClazz) {
      deleteThisMainClazz = mainClazz;
      
   }

   /** 
    * get the superclass that it extends 
    */
   private ICompilationUnit getExtendsClass(ICompilationUnit compilationUnit) {
      IType classType = getType(compilationUnit);

      ICompilationUnit superClass = null;
      try {
         String superClassName = classType.getSuperclassName();
         String[][] superResolution = classType.resolveType(superClassName);
         IJavaElement superE = JavaCore.create(superResolution[0][0] + "." + superResolution[0][1]);
//         superClass = typeHierarchy.getSuperclass(primaryType).getCompilationUnit();
      } catch (Exception e) {
         logger_.error("Error finding super clazz", e);
      }

//      return superClass;
      return deleteThisMainClazz;
   }

   /**
    * Get the interfaces it implements directly 
    */
   private ICompilationUnit[] getInterfaceUnits(ICompilationUnit compilationUnit) {
      IType classType = getType(compilationUnit);

      String[] superInterfaces = null;
      try {
         superInterfaces = classType.getSuperInterfaceNames();
      } catch (Exception e) {
      }
      
      ICompilationUnit[] superInterfaceUnits = new ICompilationUnit[superInterfaces.length];

      for (int i = 0; i < superInterfaces.length; i++) {
//         superInterfaceUnits[i] = superInterfaces[i].getCompilationUnit();
      }

//      return superInterfaceUnits;
      return new ICompilationUnit[] { deleteThisMainClazz, deleteThisMainClazz};
   }

   private IType getType(ICompilationUnit compilationUnit) {
      String compilationUnitName = compilationUnit.getElementName();
      int dotPosition = compilationUnitName.indexOf('.');
      String typeName = compilationUnitName.substring(0, dotPosition);

      IType classType = (IType)(compilationUnit.getType(typeName));
      return classType;
   }

   /**
    * We have selected a java file and are opening a UML class diagram editor against it
    */
   private boolean isJavaFile(IEditorInput input) {
      return input.getName().endsWith(JAVA_EXTENSION);
   }

   /** 
    * We have chosen a class diagram file.
    */
   private boolean isClazzDiagram(IEditorInput input) {
      return input.getName().endsWith(CLAZZ_EXTENSION);
   }

   protected void initializeGraphicalViewer() {
      GraphicalViewer viewer = getGraphicalViewer();
      viewer.addDropTargetListener(new ClazzDropListener(getGraphicalViewer()));

      viewer.setContents(diagramModel);
   }

   protected void createGraphicalViewer(Composite parent) {
      rulerComp_ = new RulerComposite(parent, SWT.NONE);
      super.createGraphicalViewer(rulerComp_);
      rulerComp_.setGraphicalViewer((ScrollingGraphicalViewer) getGraphicalViewer());
   }

   protected FigureCanvas getEditor() {
      return (FigureCanvas) getGraphicalViewer().getControl();
   }

   protected void configureGraphicalViewer() {
      super.configureGraphicalViewer();
      GraphicalViewer viewer = getGraphicalViewer();
      viewer.setEditPartFactory(new ClazzDiagramEditPartFactory());
   }

   public void doSave(IProgressMonitor monitor) {
   }

   public void doSaveAs() {
   }

   public boolean isDirty() {
      return false;
   }

   public boolean isSaveAsAllowed() {
      return false;
   }

   protected PaletteRoot getPaletteRoot() {
      PaletteRoot root = new PaletteRoot();

      PaletteGroup toolGroup = new PaletteGroup("New tool group");

      //add a selection tool entry
      ToolEntry tool = new SelectionToolEntry();
      toolGroup.add(tool);

      //set the selection tool as default
      root.setDefaultEntry(tool);

      //add a marquee tool entry
      toolGroup.add(new MarqueeToolEntry());

      //add the class diagram drawer
      PaletteDrawer drawer = createClassDiagramDrawer();

      root.add(toolGroup);
      root.add(drawer);

      return root;
   }

   private PaletteDrawer createClassDiagramDrawer() {
      PaletteDrawer drawer = new PaletteDrawer(DRAWER_NAME);

      //new class creation tool
      CreationToolEntry creationEntry = createToolEntry("newModel.gif",
            "class", "create a new class", new ClazzModelFactory());

      drawer.add(creationEntry);

      //new Line Connection
      ConnectionCreationToolEntry connxCreationEntry = createConnectionToolEntry(
            "newConnection.gif", "association", "create a new association",
            new SimpleFactory(LineConnectionModel.class));

      drawer.add(connxCreationEntry);

      //new arrow connection
      ConnectionCreationToolEntry extendsConnxCreationEntry = createConnectionToolEntry(
            "arrowConnection.gif", "extends",
            "create a new extends connection", new SimpleFactory(
                  ArrowConnectionModel.class));

      drawer.add(extendsConnxCreationEntry);


      //new arrow connection
      ConnectionCreationToolEntry implementsConnxCreationEntry = createConnectionToolEntry(
            "implementsConnection.gif", "implements",
            "create a new implements connection", new SimpleFactory(
                  ImplementsConnectionModel.class));

      drawer.add(implementsConnxCreationEntry);

      return drawer;
   }

   private CreationToolEntry createToolEntry(String iconFilename, String label,
         String shortDesc, CreationFactory modelCreationFactory) {
      ImageDescriptor descriptor = IconImage.getIconImage(iconFilename);

      CreationToolEntry creationEntry = new CreationToolEntry(label, shortDesc,
            modelCreationFactory, descriptor, descriptor);

      return creationEntry;
   }

   private ConnectionCreationToolEntry createConnectionToolEntry(
         String iconFilename, String label, String shortDesc,
         CreationFactory modelCreationFactory) {
      ImageDescriptor descriptor = IconImage.getIconImage(iconFilename);

      ConnectionCreationToolEntry creationEntry = new ConnectionCreationToolEntry(
            label, shortDesc, modelCreationFactory, descriptor, descriptor);

      return creationEntry;
   }

}