package com.alwara.uml.clazz.controller.command;

import java.util.Collections;
import java.util.List;

import org.eclipse.gef.commands.Command;

import com.alwara.uml.clazz.model.ClazzDiagramModel;
import com.alwara.uml.clazz.model.ClazzModel;
import com.alwara.uml.model.AbstractConnectionModel;

/**
 * Virender Singh Dogra
 */
public class ClazzDeleteCommand extends Command {
   private ClazzDiagramModel clazzDiagramModel_;
   private ClazzModel clazzModel_;
   private List sourceConnections_ = Collections.EMPTY_LIST;
   private List targetConnections_ = Collections.EMPTY_LIST;

   public void execute() {
      sourceConnections_ = clazzModel_.getModelSourceConnections();
      targetConnections_ = clazzModel_.getModelTargetConnections();
      
      disconnectClazz(sourceConnections_);
      disconnectClazz(targetConnections_);
      
      clazzDiagramModel_.removeClazz(clazzModel_);
   }

   /**
    * Disconnect the class from any connections made to and from it.
    */
   private void disconnectClazz(List connections) {
      for (int i = 0; i < connections.size(); i++) {
         AbstractConnectionModel model = (AbstractConnectionModel) connections.get(i);
         model.detachSource();
         model.detachTarget();
      }
   }

   public void setContentsModel(Object model) {
      clazzDiagramModel_ = (ClazzDiagramModel) model;
   }

   public void setClazzModel(Object model) {
      clazzModel_ = (ClazzModel) model;
   }

   public void undo() {
      clazzDiagramModel_.addClazz(clazzModel_);
      
      connectClazz(sourceConnections_);
      connectClazz(targetConnections_);
      
      sourceConnections_.clear();
      targetConnections_.clear();
   }

   /**
    * Connect the class back to any connections that were deleted.
    */
   private void connectClazz(List connections) {
      for (int i = 0; i < connections.size(); i++) {
         AbstractConnectionModel model = (AbstractConnectionModel) connections.get(i);
         model.attachSource();
         model.attachTarget();
      }
   }

 }
