/*
 * Created on Jan 1, 2005
 */
package com.alwara.uml.clazz.model;

import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jdt.core.ICompilationUnit;

import com.alwara.uml.model.AbstractNodeModel;
import com.alwara.uml.utils.InspectJavaElementUtils;

/**
 * @author Virender Singh Dogra
 */
public class ClazzModel extends AbstractNodeModel {
   /** Comment for <code>serialVersionUID</code> */
   private static final long serialVersionUID = 1L;

   private ICompilationUnit clazz_;

   private String name_ = "new class";

   private ClazzDiagramModel diagramModel_;

   private Rectangle constraint_;

   private PropertyContainerModel propertyContainerModel_;

   private MethodContainerModel methodContainerModel_;

   public ClazzModel(String clazzName) {
      this.name_ = clazzName;
   }

   /**
    * Constructor.
    * @param clazz
    */
   public ClazzModel(ICompilationUnit clazz) {
      clazz_ = clazz;

      propertyContainerModel_ = new PropertyContainerModel(this, clazz);
      methodContainerModel_ = new MethodContainerModel(this, clazz);

      if (clazz != null) {
         setName(clazz.findPrimaryType().getElementName());
      }
   }

   /**
    * @return the class
    */
   public ICompilationUnit getClazz() {
      return clazz_;
   }

   /**
    * @return the name of the class
    */
   public String getName() {
      return name_;
   }

   /**
    * @return get children of the model
    */
   public List getModelChildren() {
      // TODO Auto-generated method stub
      return null;
   }

   public void setName(String name) {
      this.name_ = name;
   }

   public void setDiagramModel(ClazzDiagramModel diagramModel) {
      this.diagramModel_ = diagramModel;
   }

   /**
    * @return the rectangle constraint
    */
   public Rectangle getConstraint() {
      return this.constraint_;
   }

   public void setConstraint(Rectangle constraint) {
      constraint_ = constraint;
      firePropertyChange(CONSTRAINT, null, constraint);
   }

   public boolean isInterfaceOrAbstract() {
      return InspectJavaElementUtils.isInterfaceOrAbstract(clazz_);
   }

   public boolean isClazz() {
      return InspectJavaElementUtils.isClazz(clazz_);
   }

   public boolean isAbstract() {
      return InspectJavaElementUtils.isAbstract(clazz_);
   }

   public PropertyContainerModel getPropertyContainerModel() {
      return propertyContainerModel_;
   }

   public MethodContainerModel getMethodContainerModel() {
      return methodContainerModel_;
   }
}