/*
 * Created on Jan 3, 2005
 */
package com.alwara.uml.clazz.view;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.dnd.AbstractTransferDropTargetListener;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;

import com.alwara.uml.clazz.model.ClazzModelFactory;
import com.alwara.uml.utils.InspectJavaElementUtils;

/**
 * @author Virender Singh Dogra
 */
public class ClazzDropListener extends AbstractTransferDropTargetListener {

   private ClazzModelFactory clazzModelFactory_ = new ClazzModelFactory();

   public ClazzDropListener(EditPartViewer viewer, Transfer xfer) {
      super(viewer, xfer);
   }

   public ClazzDropListener(EditPartViewer viewer) {
      super(viewer, FileTransfer.getInstance());
   }

   /**
    * {@inheritDoc}
    * Setting the current drop location on the request.
    */
   protected void updateTargetRequest() {
      ((CreateRequest) getTargetRequest()).setLocation(getDropLocation());
   }

   protected Request createTargetRequest() {
      CreateRequest request = new CreateRequest();
      request.setFactory(clazzModelFactory_);
      return request;
   }

   /**
    * {@inheritDoc}
    * Now since we'll be dragging files from the Navigator (or any file explorer), we
    * want to ensure that the file won't be deleted. To do this, override handleDragOver() and set
    * the drop detail to DND.DROP_COPY. If we leave it as the default (DND.DROP_MOVE), after the
    * drop the drag source will assume the file is somewhere else and remove the original. We want
    * the file to stay right where it is.
    */
   protected void handleDragOver() {
      getCurrentEvent().detail = DND.DROP_COPY;
      super.handleDragOver();
   }

   protected void handleDrop() {
      String fullFileName = ((String[]) getCurrentEvent().data)[0];

      ICompilationUnit javaElement = InspectJavaElementUtils.getCompilationUnit(fullFileName);
      
      clazzModelFactory_.setClazz(javaElement);
      super.handleDrop();
   }

}