/*
 * Created on Jan 1, 2005
 */
package com.alwara.uml.clazz.view;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.swt.graphics.Color;

import com.alwara.uml.clazz.model.MethodContainerModel;
import com.alwara.uml.view.CompartmentFigureBorder;
import com.alwara.uml.view.IconImage;

/**
 * @author Virender Singh Dogra
 */
public class MethodContainerFigure extends Figure {
   public final static Color containerColor = new Color(null, 252, 254, 252); 
   private final MethodContainerModel methodContainerModel_;

   /**
    * @param methodContainerModel
    */
   public MethodContainerFigure(MethodContainerModel methodContainerModel) {
      this.methodContainerModel_ = methodContainerModel;

      ToolbarLayout layout = new ToolbarLayout();
      layout.setSpacing(2);
      layout.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
      setLayoutManager(layout);

      if (methodContainerModel.getMethodList().length > 0) {
         try {
            IMethod[] methodArray =  methodContainerModel.getMethodList();
            for (int i = 0; i < methodArray.length; i++) {
               IMethod method = methodArray[i];
               Label label = new Label(getMethodSignature(method),
                        IconImage.getFittingImage(method));
               label.setLabelAlignment(Label.LEFT);
               add(label);
            }
         }
         catch (JavaModelException e) {
            e.printStackTrace();
         }
      }
      else { //Atleast show an empty container if no properties found
         Label label = new Label(" ");
         add(label);
      }

      setBorder(new CompartmentFigureBorder());
      setBackgroundColor(containerColor);
      setOpaque(true);
   }

   private String getMethodSignature(IMethod method) throws JavaModelException {
      String mainMethod = Signature.toString(method.getSignature(), method.getElementName(), null, false, false);
      
      return mainMethod + " : " + getReturnType(method);
   }

   private String getReturnType(IMethod method) throws JavaModelException {
      return Signature.toString(method.getReturnType());
   }

 }