/*
 * Created on Jan 23, 2005
 */
package com.alwara.uml.controller;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import com.alwara.uml.controller.editpolicy.UMLBendpointEditPolicy;
import com.alwara.uml.controller.editpolicy.UMLConnectionEditPolicy;
import com.alwara.uml.controller.editpolicy.UMLConnectionEndpointEditPolicy;
import com.alwara.uml.model.AbstractConnectionModel;

/**
 * @author Virender Singh Dogra
 */
public abstract class AbstractConnectionWithListenerEditPart 
      extends AbstractConnectionEditPart 
      implements PropertyChangeListener {
   
   public void activate() {
      super.activate();
      getListeningModel().addPropertyChangeListener(this);
   }

   public void deactivate() {
      getListeningModel().removePropertyChangeListener(this);
      super.deactivate();
   }

   private AbstractConnectionModel getListeningModel() {
      return (AbstractConnectionModel) getModel();
   }

   public void propertyChange(PropertyChangeEvent evt) {
      String property = evt.getPropertyName();

      if (AbstractConnectionModel.CHILD.equals(property)) {
         refreshChildren();
      }
      else if (AbstractConnectionModel.SOURCE_CONNECTION.equals(property)) {
         refreshSourceConnections();
      }
      else if (AbstractConnectionModel.TARGET_CONNECTION.equals(property)) {
         refreshTargetConnections();
      }
      else if (AbstractConnectionModel.BEND_POINT.equals(property)) {
         refreshBendPoints();
      }
      
      ((GraphicalEditPart) (getViewer().getContents())).getFigure().revalidate();

   }
   
   protected void refreshBendPoints() {
      List bendpoints = ((AbstractConnectionModel) getModel()).getBendpoints();
      List constraint = new ArrayList();

      for (int i = 0; i < bendpoints.size(); i++) {
         constraint.add(new AbsoluteBendpoint((Point) bendpoints.get(i)));
      }

      getConnectionFigure().setRoutingConstraint(constraint);
   }

   protected IFigure createFigure(){
      PolylineConnection connection = (PolylineConnection) super.createFigure();
      connection.setConnectionRouter(new BendpointConnectionRouter());
      return connection;
  }
   

   protected void refreshVisuals() {
     refreshBendPoints();
   }
   
   protected void createEditPolicies() {
    installEditPolicy(EditPolicy.CONNECTION_ROLE,
                      new UMLConnectionEditPolicy());

    installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
                      new UMLConnectionEndpointEditPolicy());
    
    installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
                      new UMLBendpointEditPolicy());
      
  }

}