/*
 * Created on Jan 2, 2005
 */
package com.alwara.uml.controller;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import com.alwara.uml.model.AbstractModel;
import com.alwara.uml.model.AbstractNodeModel;

/**
 * @author Virender Singh Dogra
 * An abstract EditPart implementation which is property aware and responds to
 * PropertyChangeEvents fired from the model
 */
public abstract class EditPartWithListener extends AbstractGraphicalEditPart implements PropertyChangeListener {

   public void activate() {
      super.activate();
      getListeningModel().addPropertyChangeListener(this);
   }

   public void deactivate() {
      getListeningModel().removePropertyChangeListener(this);
      super.deactivate();
   }
   
   /** @return the model object that is being listened to. */
   private AbstractModel getListeningModel() {
      AbstractModel listeningModel = (AbstractModel) getModel();
      return listeningModel;
   }
   
   public void propertyChange(PropertyChangeEvent evt) {
      String property = evt.getPropertyName();

      if (AbstractModel.CHILD.equals(property)) {
//         handleChildChange(evt);
         refreshChildren();
      }
      else if (AbstractModel.CONSTRAINT.equals(property)) {
         //TODO: How is this working
         refreshVisuals();
      }
      else if (AbstractNodeModel.SOURCE_CONNECTION.equals(property)) {
         refreshSourceConnections();
      }
      else if (AbstractNodeModel.TARGET_CONNECTION.equals(property)) {
         refreshTargetConnections();
      }
   }

   /**
    * called when child added or removed
    */
   protected void handleChildChange(PropertyChangeEvent evt) {

      Object newValue = evt.getNewValue();
      Object oldValue = evt.getOldValue();

      if (!((oldValue != null) ^ (newValue != null))) {
         throw new IllegalStateException("Exactly one of old or new values must be non-null for CHILD event");
      }

      if (newValue != null) {
         //add new child
         EditPart editPart = createChild(newValue);
         int modelIndex = getModelChildren().indexOf(newValue);
         addChild(editPart, modelIndex);
      }
      else {
         List childrenList = getChildren();

         EditPart partToRemove = null;
         for (Iterator iter = childrenList.iterator(); iter.hasNext();) {
            EditPart part = (EditPart) iter.next();
            if (part.getModel() == oldValue) {
               partToRemove = part;
               break;
            }
         }

         if (partToRemove != null)
            removeChild(partToRemove);
      }
   }

}