/*
 * Created on Jan 2, 2005
 */
package com.alwara.uml.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;


/**
 * Provides base class support for model objects to participate in event handling framework
 * @author Virender Singh Dogra
 */
public abstract class AbstractModel implements Serializable {

   public static final String 
            CHILD = "CHILD",
            CONSTRAINT = "CONSTRAINT",
            SOURCE_CONNECTION = "SOURCE_CONNECTION",
            TARGET_CONNECTION = "TARGET_CONNECTION";
   
   protected transient PropertyChangeSupport listeners_ = new PropertyChangeSupport(this);


   protected AbstractModel() {
   }

   public void addPropertyChangeListener(PropertyChangeListener listener) {
      listeners_.addPropertyChangeListener(listener);
   }

   public void removePropertyChangeListener(PropertyChangeListener listener) {
      listeners_.removePropertyChangeListener(listener);
   }

   protected void firePropertyChange(String prop, Object old, Object newValue) {
      listeners_.firePropertyChange(prop, old, newValue);
   }

   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      listeners_ = new PropertyChangeSupport(this);
   }

}