/*
 * Created on Jan 5, 2005
 */
package com.alwara.uml.view;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jdt.internal.ui.viewsupport.JavaElementImageProvider;
import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaElementImageDescriptor;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import com.alwara.uml.clazz.model.ClazzModel;
import com.alwara.uml.model.AbstractModel;
import com.alwara.uml.utils.Logger;

/**
 * @author Virender Singh Dogra
 */
public class IconImage {
   static Logger logger_ = Logger.getLogger(IconImage.class);

   private static final String PLUGIN_IN = "com.alwara.uml";
   private static final String ICONS_PATH = "icons/";
   
   public static final Image 
      NON_PROJECT_CLASS = JavaPluginImages.get( "org.eclipse.jdt.ui.classfo_obj.gif" ),
      PROJECT_CLASS = JavaUI.getSharedImages().getImage( ISharedImages.IMG_OBJS_CLASS ),
      PROJECT_INTERFACE = JavaUI.getSharedImages().getImage( ISharedImages.IMG_OBJS_INTERFACE ),
      NON_PROJECT_INTERFACE = JavaPluginImages.get( "org.eclipse.jdt.ui.intf_obj.gif" ),
      NON_PROJECT_PACKAGE = JavaUI.getSharedImages().getImage( ISharedImages.IMG_OBJS_EMPTY_PACKAGE ),
      PROJECT_PACKAGE = JavaUI.getSharedImages().getImage( ISharedImages.IMG_OBJS_PACKAGE ),
      FIELD_PRIVATE = JavaPluginImages.get(JavaPluginImages.IMG_FIELD_PRIVATE),
      FIELD_DEFAULT = JavaPluginImages.get(JavaPluginImages.IMG_FIELD_DEFAULT),
      FIELD_PROTECTED = JavaPluginImages.get(JavaPluginImages.IMG_FIELD_PROTECTED),
      FIELD_PUBLIC = JavaPluginImages.get(JavaPluginImages.IMG_FIELD_PUBLIC),
      METHOD_CONSTRUCTOR = JavaPluginImages.DESC_OVR_CONSTRUCTOR.createImage(),
      METHOD_PRIVATE = JavaPluginImages.get(JavaPluginImages.IMG_MISC_PRIVATE),
      METHOD_DEFAULT = JavaPluginImages.get(JavaPluginImages.IMG_MISC_DEFAULT),
      METHOD_PROTECTED = JavaPluginImages.get(JavaPluginImages.IMG_MISC_PROTECTED),
      METHOD_PUBLIC = JavaPluginImages.get(JavaPluginImages.IMG_MISC_PUBLIC);

   /**
    * @return an icon image
    */
   public static Image getFittingImage(AbstractModel model) {
      Image newImage = null;
      
      if (model instanceof ClazzModel){
         ClazzModel clazzModel = (ClazzModel)model; 
         if (clazzModel.isInterfaceOrAbstract()) {
            newImage = PROJECT_INTERFACE;
         }
         if (clazzModel.isAbstract()) {
            newImage = PROJECT_CLASS;
         }
         else if (clazzModel.isClazz()) {
            newImage = PROJECT_CLASS;
         }
      }
      return newImage;
   }

   /**
    * @param method
    * @return a image for the method
    */
   public static Image getFittingImage(IMethod method) {
      Image returnImage = null;
      
      try {
         int flags = method.isConstructor() ? JavaElementImageDescriptor.CONSTRUCTOR : 0;
         returnImage = getElementImageLabel(method, flags);
      }
      catch (JavaModelException e) {
         logger_.error("While getting modifier", e);
      }
      
      return returnImage;
   }

   /**
    * @param field
    * @return the image for the method
    */
   public static Image getFittingImage(IField field) {
      Image returnImage = null;
      
      try {
         returnImage = getElementImageLabel(field, 0);
      }
      catch (JavaModelException e) {
         logger_.error("While getting modifier", e);
      }
      
      return returnImage;
   }

   /**
    * @param member
    * @param additionalFlags
    * @return an image label for this element
    * @throws JavaModelException
    */
   private static Image getElementImageLabel(IMember member, int additionalFlags) throws JavaModelException {
      Image returnImage;
      int flags = additionalFlags 
               | JavaElementImageProvider.OVERLAY_ICONS
               | (Flags.isFinal(member.getFlags()) ? JavaElementImageDescriptor.FINAL : 0)
               | (Flags.isStatic(member.getFlags()) ? JavaElementImageDescriptor.STATIC : 0)
               | (Flags.isAbstract(member.getFlags()) ? JavaElementImageDescriptor.ABSTRACT : 0)
               | (Flags.isSynchronized(member.getFlags()) ? JavaElementImageDescriptor.SYNCHRONIZED : 0)
              ;
      
      returnImage = new JavaElementImageProvider().getImageLabel(member, flags);
      return returnImage;
   }

   public static ImageDescriptor getIconImage(String iconFilename) {
      ImageDescriptor descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_IN, ICONS_PATH + iconFilename);
      return descriptor;
   }
   
}